/**
 * @fileOverview
 * @name dialplan.js
 * @author Ketlin Pedron
 */

/**
 * @typedef {Object} DialplanController~DialplanParams - Dialplan parameters.
 * @property {String} textfile - Plaintext to be saved in extensions.conf.
 */

/**
 * @class DialplanController
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @property {DialplanController~DialplanParams}.
 */
angular.module('ag561app').controller('DialplanController', ['$http', '$log', '$location',
                                   function($http, $log, $location) {
  var url_dialplan = '/api/config/dialplan';
  var ctrl = this;
  ctrl.loading = false;
  ctrl.invalid_config = false;

  ctrl.called_pattern_list = ['[2-8]XXXXXXX',   // FIXO
                              '9XXXXXXXX',  // Celular
                              '0ZXZZ[2-8]XXXXXXX',  // DDD FIXO
                              '0ZXZZ9XXXXXXXX',   // DDD Celular
                              '00Z.',   // Ligacao Internacional
                              '9090[2-8]XXXXXXX',   // Chamada a cobrar para fixo
                              '90909XXXXXXXX',   // Chamada a cobrar para celular,
                              '90ZXZZ[2-8]XXXXXXX',  // Chama a cobrar longa distancia para Fixo
                              '90ZXZZ9XXXXXXXX',  // Chama a cobrar longa distancia para Celular
                              '19X',    // Ligacao de emergencia
                              '0800X.',   // Ligacao gratuita para 0800
                              '0300XXXXXXX',  // Ligacao para 0300
                              '0500XXXXXXX',  // Ligacao para 0500
                              '0900XXXXXXX',  // Ligacao para 0900
                              '1[0-8].'  // Ligacao para numeros especiais. Ex.: Menu de servicos da operadora
                              ];

  ctrl.callerid_pattern_list = ctrl.called_pattern_list;

  ctrl.mfcr2_options = ['national_subscriber',
                        'national_priority_subscriber',
                        'international_subscriber',
                        'international_priority_subscriber',
                        'collect_call'
                        ];

  ctrl.isdn_options = ['no',
                       'yes'
                      ];

  /**
   * Clear alert message from web page, but do this only
   * if we aren't with some error. We just want to erase
   * the message of danger if all the variables are correct.
   *
   * @function clear_alert_messages
   * @memberof DialplanController
   */
  ctrl.clear_alert_messages = function() {
    if (!ctrl.invalid_config) {
      ctrl.alert_danger = false;
      ctrl.alert_success = false;
    }
  };

  /**
   * Get text from sip.conf file and gets too
   * pointers to rules and peers.
   *
   * @function get
   * @memberof DialplanController
   */
  ctrl.get = function() {
    // GET Dialplan config
    $http({
      method: 'GET',
      url: url_dialplan
      }).then(function(response) {
        // First function handles success
        ctrl.rules = response.data.rules;
        ctrl.peers = response.data.peers;
        ctrl.failover_peers = response.data.failover_peers;
      }, function(response) {
        // Second function handles error
        ctrl.error = response.data;
      });
  };

  /**
   * Save the new configuration.
   * If some error is happening then show in
   * web page.
   *
   * @function save
   * @memberof DialplanController
   */
  ctrl.save = function() {

    var i = 0;

    while(ctrl.rules[i]) {
      // Everything is ok, then we need remove the variables that can not go to API
      delete ctrl.rules[i].view;
      delete ctrl.rules[i].invalid_called_pattern;
      delete ctrl.rules[i].invalid_callerid_pattern;
      delete ctrl.rules[i].invalid_outgoing_called_pattern;
      delete ctrl.rules[i].invalid_outgoing_callerid_pattern;
      delete ctrl.rules[i].invalid_answer_timeout;

      i++;
    }

    data = {'rules': ctrl.rules};

    ctrl.loading = true;
    $http({
      method: 'PUT',
      url: url_dialplan,
      data: data
      }).then(function(response) {
        // First function handles success
        ctrl.loading = false;
        ctrl.alert_success = true;
        ctrl.get();
      }, function(response) {
        // Second function handles error
        ctrl.loading = false;
        ctrl.alert_danger = true;
        ctrl.error = response.data;
      });
  };

  function find_rule_name(name) {

    var i;
    for (i = 0; i < ctrl.rules.length; i++) {
      if (ctrl.rules[i].name == name)
        return ctrl.rules[i];
    }

    return false;
  };

  /**
   * Add a new rule to object 'ctrl.rules'.
   * But attention, to add it in configuration file it's
   * necessary that the user click in button 'Save and apply' in
   * web page.
   *
   * @function add_rule
   * @memberof DialplanController
   */
  ctrl.add_rule = function() {
    var i = 0;
    var rule_name;

    do {
      i++;
      rule_name = 'rule' + i.toString();
    } while (find_rule_name(rule_name));

    // Start default configuration
    ctrl.newrule = {'called_pattern': 'X.',
                    'source_peer': Object.keys(ctrl.peers)[0],
                    'outgoing_called': '{}',
                    'outgoing_callerid': '{}',
                    'answer_timeout': '90',
                    'failover_peer': 'disabled',
                    'name':  rule_name};

    ctrl.newrule.view = true;
    ctrl.rules.push(ctrl.newrule);

  };

   /**
   * Copy a configuration from a specific rule
   * to a new rule. But the rule name is
   * different.
   *
   * @function clone_rule
   * @memberof DialplanController
   */
  ctrl.clone_rule = function(rule) {
    /* */
    let newrule = angular.copy(rule);
    let i = 0;

    do {
      i++;
      rule_name = newrule.name + i.toString();
    } while (find_rule_name(rule_name));

    newrule.name = rule_name;
    newrule.view = !newrule.view;

    ctrl.rules.push(newrule);
  };


  /**
   * Remove the object previous save in variable 'rule_toremove'
   * from main object 'ctrl.rules'.
   * But attention, to remove it in configuration file it's
   * necessary that the user click in button 'Save and apply' in
   * web page.
   *
   * @function remove_rule
   * @memberof DialplanController
   */
  ctrl.remove_rule = function() {
    ctrl.rules.splice(ctrl.rule_toremove, 1);
  };

  /**
   * Save a object to remove in next step.
   *
   * @function save_rule_to_remove
   * @memberof DialplanController
   */
  ctrl.save_rule_to_remove = function(array_index) {
    ctrl.rule_toremove = array_index;
  };

  /**
   * Verify if some configuration is invalid.
   * Update the variable 'alert_danger' and
   * invalid_config.
   *
   * @function _check_invalid_config
   * @memberof DialplanController
   */
  _check_invalid_config = function() {
    ctrl.invalid_config = false;
    for (var i = 0; i < ctrl.rules.length; i++) {
      var rule = ctrl.rules[i];
      if (rule.invalid_called_pattern || rule.invalid_callerid_pattern ||
          rule.invalid_outgoing_called_pattern || rule.invalid_outgoing_callerid_pattern ||
          rule.invalid_answer_timeout || rule.invalid_rule_name) {
        ctrl.invalid_config = true;
        break;
      }
    }
    ctrl.alert_danger = ctrl.invalid_config;
  };

  /**
   * Check if value to 'Called Pattern' is correct.
   * This string can't be empty.
   * If isn't correct then print it in web page.
   *
   * @function validateCalledPattern
   * @memberof DialplanController
   */
  var pattern_regex = /^\+?([\dNXZ*#]|\[[\d*#\+]+(-[\d*#\+]+)?\])+[.!]?$/;

  ctrl.validateCalledPattern = function(rule) {
    rule.invalid_called_pattern = !pattern_regex.test(rule.called_pattern)
    _check_invalid_config();
  };

  /**
   * Check if value to 'Callerid Pattern' is correct.
   * This string can be empty.
   * But if string isn't empty then check the string
   * with the regex, if isn't correct then print it
   * in web page.
   *
   * @function validateCallerIDPattern
   * @memberof DialplanController
   */
  ctrl.validateCallerIDPattern = function(rule) {
    rule.invalid_callerid_pattern = rule.callerid_pattern.length > 0 && !pattern_regex.test(rule.callerid_pattern);
    _check_invalid_config();
  };

  /**
   * Check if value to 'Outgoing Called' is correct.
   * If isn't correct then print it in web page.
   * We can accept a empty string, but the default
   * will '{}', the API will do the default.
   *
   * @function validateOutgoingCalled
   * @memberof DialplanController
   */
  var outgoing_regex = /^\+?([\d#*{}:\-])*$/;

  ctrl.validateOutgoingCalled = function(rule) {
    rule.invalid_outgoing_called_pattern = !outgoing_regex.test(rule.outgoing_called);
    _check_invalid_config();
  };

  /**
   * Check if value to 'Outgoing CallerID' is correct.
   * If isn't correct then print it in web page.
   * We can accept a empty string, but the default
   * will '{}', the API will do the default.
   *
   * @function validateOutgoingCallerID
   * @memberof DialplanController
   */
  ctrl.validateOutgoingCallerID = function(rule) {
    rule.invalid_outgoing_callerid_pattern = !outgoing_regex.test(rule.outgoing_callerid);
    _check_invalid_config();
  };

  /**
   * Check if valut to 'Answer Timeout' is correct.
   * Accept only numbers, we don't have a maximum value.
   * We don't accept a empty string.
   * If isn't correct then print it in web page.
   *
   * @function validateAnswerTimeout
   * @memberof DialplanController
   */
  ctrl.validateAnswerTimeout = function(rule) {
    regex = /^[0-9]+([0-9]+)?$/;

    rule.invalid_answer_timeout = !regex.test(rule.answer_timeout);
    _check_invalid_config();
  };

  /**
   * When we add a new rule it's necessary that the user
   * attach information about the rule name.
   * We don't accept.
   *
   * @function validateRuleName
   * @memberof DialplanController
   */
  ctrl.validateRuleName = function(rule) {
    regex = /^([\w])+$/;

    rule.invalid_rule_name = false;
    if (!regex.test(rule.name)) {
      /* Invalid rule name */
      rule.invalid_rule_name = true;
    } else {
      // Check if this name already exists in another rule
      var count = 0;
      for (var i = 0; i < ctrl.rules.length; i++) {
        if (ctrl.rules[i].name == rule.name)
          count++;
      }

      if (count > 1) {
        rule.invalid_rule_name = true;
      }
    }
    _check_invalid_config();
  };

  ctrl.get_signalling = function(peer) {

    // In some situations it's possible that the destination peer
    // is a empty string, then we don't need to get the signalling
    if (peer == null) {
      return "";
    }
    for(var peer_name in ctrl.peers) {
      if (peer_name == peer) {
        for (let [key, value] of Object.entries(ctrl.peers[peer_name])) {
          if (key == "signalling") {
            // In this situation we have a TDM destination peer, then read
            // about its signalling value.
            return value;
          }
        }
      }
    }
  };

  ctrl.get();

}]);
